/* mass storage buffer handling */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <util/atomic.h>

#include "buffer.h"

static buffer_block block_buffer[NUM_BUFFERS];
static buffer_state block_states[NUM_BUFFERS];

bufnum_t
get_first_free_buffer (void)
{
  unsigned char i;
  for (i = 0; i<NUM_BUFFERS; i++) {
    if (!block_states[i].used)
      return i;
  }
  return -1;
}

buffer_block *
get_free_buffer_pin (void)
{
  bufnum_t b;
  do {
    b = get_first_free_buffer();
    if (b<0) return 0;
    ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
    {
      if (!block_states[b].used) {
	block_states[b].used = 1;
	return &(block_buffer[b]);
      }
    }
  } while (1);
}

bufnum_t
find_oldest_buffer (uint8_t can_be_dirty)
{
  bufnum_t i;
  bufnum_t best = -1;
  uint8_t bestcreated = UINT8_MAX;
  for (i = 0; i < NUM_BUFFERS; i++) {
    if (block_states[i].io_in_progress || block_states[i].update_in_progress)
      continue;
    if (can_be_dirty || !block_states[i].dirty) {
      if (block_states[i].created < bestcreated) {
	best = i; bestcreated = block_states[i].created;
      }
    }
  }
  return best;
}
