/* interface to mass-storage buffer handling */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef BUFFER_H
#define BUFFER_H

#include <stdint.h>

/* size of a single buffer */
#define BUFSIZE (512)
#define NUM_BUFFERS (4)

typedef int8_t bufnum_t;

/* possible mass storage sizes with BUFSIZE=512:
 * char  (8bit)  128kB
 * short (16bit)  32MB
 * long  (32bit)   2TB
 */
typedef uint16_t blocknum_t;

typedef uint8_t buffer_block [BUFSIZE];

typedef struct {
  blocknum_t blocknum;
  uint8_t created; /* for LRU */
  unsigned char used:1;
  unsigned char dirty:1;
  unsigned char io_in_progress:1;
  unsigned char update_in_progress:1; /* the buffer is being updated and should
				       * *NOT* be written out now */
} buffer_state;

#endif
