/* db101 mock nmea event handler */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <avr/pgmspace.h>
#include <terminal_lib.h>
#include <backlight_driver.h>
#include "../events.h"
#include "../nmea_reader.h"

extern TERM_terminal_t term;


uint8_t put_nmea_event (nmea_data_t * p)
{
  static gps_time_t last_time;
  gps_time_t time = p->time;
  char buffer[30];
  uint8_t hour, min, sec;

  if (p->status.time && p->status.latitude && p->status.longitude) {
    uint32_t latminutes = p->latitude >> 16;
    uint8_t deglat, minlat;
    uint32_t lonminutes = p->longitude >> 16;
    uint8_t deglon, minlon;
    
    if (time.seconds == last_time.seconds) return 1;
    time = last_time = time;

    sec = time.seconds % 60; time.seconds /= 60;
    min = time.seconds % 60; time.seconds /= 60;
    hour = time.seconds;
    
    minlat = latminutes % 60; deglat = latminutes / 60;
    minlon = lonminutes % 60; deglon = lonminutes / 60;
    
    snprintf_P (buffer, sizeof(buffer),
		PSTR("%02d%02d%02d: %02d%c%02d %03d%c%02d\r\n"),
		hour, min, sec,
		deglat, p->latitude_sign ? 'S' : 'N', minlat,
		deglon, p->longitude_sign ? 'W' : 'E', minlon);
  
    TERM_ProcessString (&term, buffer);
  }

  if (p->status.hdop) {
    /* background color according to HDOP */
    uint16_t hdop = p->hdop >> 16;
    int16_t color_base = 0xFF;
    uint8_t color0;

    if (hdop < 2) {
      color_base -= hdop*8;
    } else if (hdop < 10) {
      color_base -= 16+(hdop-2)*4;
    } else if (hdop < 30) {
      color_base -= 48+(hdop-10)*2;
    } else if (hdop < 90) {
      color_base -= 88+(hdop-30);
    } else {
      color_base -= 148+(hdop-90);
      if (color_base<0) color_base = 0;
    }

    color0 = color_base;
    BACKLIGHT_SetRGB (0xFF-color0, color0, color0/2);
  }

  if (p->status.positionfix) {
    switch (p->positionfix) {
    case NPF_INVALID:
      BACKLIGHT_SetRGB (0xFF, 0, 0);
      break;
    case NPF_DEADRECKONING:
      BACKLIGHT_SetRGB (0xFF, 0, 0x80);
      break;
    default:
      break;
    }
  }
  return 0;
}
