/* interface to event queue */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef EVENTS_H
#define EVENTS_H

#include "gps.h"

extern volatile unsigned char event_queue_cnt;

typedef enum {
  NOP,
  KBD_I2C,
  KEY,
  TIMER,
  SERIAL_IN,
  SERIAL_OUT,
  NMEA_POS,
  SD_READY
} event_type;

typedef struct event_s {
  event_type ev_type;
  union {
    /* NOP */
    /* KBD_I2C */
    struct {
      uint8_t byte0, byte1;
    } i2c;
    /* KEY */
    struct {
      uint16_t changed;
      uint16_t pressed;
    } k;
    /* TIMER */
    struct {
      unsigned char timer_id;
    } t;
    /* SERIAL_IN */
    struct {
      uint8_t byte;
    } s;
    /* SERIAL_OUT */
    /* NMEA_POS */
    gps_position p;
    /* SD_READY */
  } ev_data;
} event;

/* forward decl */
struct nmea_data_s;

/* handle Nth event in queue */
void handle_event (unsigned char n);

uint8_t put_serial_in_event (uint8_t byte);
uint8_t put_nmea_event (/*gps_time_t time, gps_date_t date,
			uint32_t lat, uint16_t latf,
			uint32_t lon, uint16_t lonf, char latsign,
			char lonsign, uint16_t height, uint16_t hdop,
			uint16_t vdop*/
			struct nmea_data_s *);
uint8_t put_kbd_i2c_event (uint8_t byte0, uint8_t byte1);
uint8_t put_key_event (uint16_t changed, uint16_t pressed);

#endif
