/* definitions for keyboard via I2C */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef KBD_I2C_H
#define KBD_I2C_H

#include <stdint.h>
#include <avr/io.h>

#define KBD_I2C_PORT PORTE
#define KBD_I2C_SCL  PORTE4
#define KBD_I2C_SDA  PORTE5
#define KBD_I2C_INT  PORTE6

#define KBD_I2C_PORT_D DDRE
#define KBD_I2C_SCL_D DDE4
#define KBD_I2C_SDA_D DDE5
#define KBD_I2C_INT_D DDE6

#define KBD_I2C_PIN  PINE
#define KBD_I2C_SCL_I PINE4
#define KBD_I2C_SDA_I PINE5
#define KBD_I2C_INT_I PINE6

/* I2C target frequency */
#define F_TWI (100000UL)

#define TCCR_BASEVAL (_BV(WGM01))
#define TCCR_CLKSRC  (_BV(CS00))

/* address of expander, bit 6 is fixed, bits 3-1 defined by inputs
 * bit 0 is R/W-bit (1 is read)
 */
#define KBD_I2C_ADDRESS (0x40)

#define KBD_READ (1)
#define KBD_WRITE (0)

#define KBD_I2C_MAXCMDSIZE (5)

/* structure defining I2C message */
typedef struct {
  uint8_t msglen; /* length of message, excluding this byte,
		   * 0 indicates end of stream */
  uint8_t msg[1]; /* message, first byte is address, bit 0 of that
		   * is 1 for reading, 0 for writing
		   * code for continuing action when msglen == 0 */
} i2c_msg_t;

#define I2C_END_INIT (1)
#define I2C_END_KBD  (2)

uint8_t kbd_i2c_init (void);

#endif
