/* LCD driver routines */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <avr/io.h>
#include <string.h>
#include <avr/interrupt.h>

#include "lcd.h"

static uint8_t lcd_buffer [LCD_PLANES][LCD_DIGITS];

uint8_t
lcd_init (void)
{
  /* internal clock, 1/3 bias, 1/4 duty, 37 segments */
  LCDCRB = _BV(LCDMUX1) | _BV(LCDMUX0)
    | _BV(LCDPM3) | _BV(LCDPM2) |/* _BV(LCDPM1) |*/ _BV(LCDPM0);
  /* prescaler=4096, div=8 */
  LCDFRR = _BV(LCDPS2) | _BV(LCDPS1) | _BV(LCDPS0)
    | _BV(LCDCD2) | _BV(LCDCD1) | _BV(LCDCD0);
  /* enable LCD and Frame interrupt */
  LCDCRA = _BV(LCDEN) | _BV(LCDIE);

  /* LCD switchport is OUT */
  LCD_SWITCHPORT_D |= _BV(LCD_SWITCHPIN_D);

  return 1;
}

void
lcd_shutdown (void)
{
  /* wait until new frame started */
  while (!(LCDCRA & _BV(LCDIF)));
  /* disable & clear interrupt, set blank */
  LCDCRA = (LCDCRA & ~_BV(LCDIE)) | _BV(LCDBL) | _BV(LCDIF);
  /* wait until next frame */
  while (!(LCDCRA & _BV(LCDIF)));
  /* disable LCD */
  LCDCRA &= ~_BV(LCDEN);
}

ISR(LCD_vect)
{
  static uint8_t lcd_current_plane;
  lcd_current_plane = (uint8_t)(lcd_current_plane+1) % LCD_PLANES;
  memcpy ((uint8_t*)&LCDDR00, lcd_buffer[lcd_current_plane], LCD_DIGITS);
  LCD_SWITCHPORT = (uint8_t)(LCD_SWITCHPORT & ~LCD_SWITCHMASK)
    | (uint8_t)(lcd_current_plane << LCD_SWITCHPIN);
}

void
lcd_put (uint8_t plane, uint8_t pos, uint8_t bits)
{
  if (plane >= LCD_PLANES) return;
  lcd_buffer[plane][pos] = bits;
}
