/// This file has been prepared for Doxygen automatic documentation generation.
/*! \file *********************************************************************
 *
 * \brief  DB101 configure UART demo application.
 *
 *         This application lets you change all settings about the UART
 *         on the DB101.
 *
 *         Number of data bits, stop bits, baud rate and interface (logic or RS232)
 *         can be adjusted. The DB101 supports both logic level and RS232 electrical
 *         interfaces.
 *
 *         NOTE: At the moment only 7 or 8 data bits are accepted
 *
 *
 * \par Application note:
 *      AVR482: DB101 Software
 *
 * \par Documentation
 *      For comprehensive code documentation, supported compilers, compiler
 *      settings and supported devices see readme.html
 *
 * \author
 *      Atmel Corporation: http://www.atmel.com \n
 *      Support email: avr@atmel.com
 *
 * $Revision: 2395 $
 * $Date: 2007-09-13 13:16:08 -0600 (to, 13 sep 2007) $
 * $URL: http://revisor.norway.atmel.com/AppsAVR8/avr482_db101_software_users_guide/tags/release_A_code_20070917/IAR/production_demo_rev_A/configuart.c $
 *
 * Copyright (c) 2006, Atmel Corporation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name of ATMEL may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 * SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include <cal.h>

#include "forms_lib.h"
#include <lcd_lib.h>
#include <backlight_driver.h>
#include <widgets_lib.h>
#include <dialog_lib.h>
#include <common.h>
#include <memblock_lib.h>
#include <termfont_lib.h>
#include <popup_lib.h>
#include <uart_driver.h>



#define CONFIG_SET_LOGIC_INTERFACE()  DDRD |= (1 << PD4); PORTD |= (1 << PD4);
#define CONFIG_SET_RS232_INTERFACE()  DDRD |= (1 << PD4); PORTD &= ~(1 << PD4);


#define BAUD_COUNT 11
#define BAUD_WIDTH 6

#define PARITY_COUNT 3
#define PARITY_WIDTH 4

#define INTERFACE_COUNT 2
#define INTERFACE_WIDTH 5

#define BUTTON_COUNT 2

static char const CAL_PGM_DEF(titleString[]) = "Configure UART";

static char const CAL_PGM_DEF(baudCaption[])      = "Baudrate:";
static char const CAL_PGM_DEF(databitCaption[])   = "Data bits:";
static char const CAL_PGM_DEF(stopbitCaption[])   = "Stop bits:";
static char const CAL_PGM_DEF(parityCaption[])    = "Parity:";
static char const CAL_PGM_DEF(interfaceCaption[]) = "Interface:";

static char const CAL_PGM_DEF(baud2400[])   =  "  2400";
static char const CAL_PGM_DEF(baud4800[])   =  "  4800";
static char const CAL_PGM_DEF(baud9600[])   =  "  9600";
static char const CAL_PGM_DEF(baud14400[])  =  " 14400";
static char const CAL_PGM_DEF(baud19200[])  =  " 19200";
static char const CAL_PGM_DEF(baud28800[])  =  " 28800";
static char const CAL_PGM_DEF(baud38400[])  =  " 38400";
static char const CAL_PGM_DEF(baud57600[])  =  " 57600";
static char const CAL_PGM_DEF(baud76800[])  =  " 76800";
static char const CAL_PGM_DEF(baud115200[]) =  "115200";
static char const CAL_PGM_DEF(baud230400[]) =  "230400";

//static char const __flash * __flash baudStrings[BAUD_COUNT] = {
static char const CAL_PGM_DEF(* CAL_PGM_DEF( baudStrings[BAUD_COUNT])) = {
	baud2400,
	baud4800,
	baud9600,
	baud14400,
	baud19200,
	baud28800,
	baud38400,
	baud57600,
	baud76800,
	baud115200,
	baud230400
};

static uint16_t const CAL_PGM_DEF(baudValues[BAUD_COUNT]) = {
	UART_CALC_UBRR_VALUE(2400),
	UART_CALC_UBRR_VALUE(4800),
	UART_CALC_UBRR_VALUE(9600),
	UART_CALC_UBRR_VALUE(14400),
	UART_CALC_UBRR_VALUE(19200),
	UART_CALC_UBRR_VALUE(28800),
	UART_CALC_UBRR_VALUE(38400),
	UART_CALC_UBRR_VALUE(57600),
	UART_CALC_UBRR_VALUE(76800),
	UART_CALC_UBRR_VALUE(115200),
	UART_CALC_UBRR_VALUE(230400)
};


static char const CAL_PGM_DEF(parityNone[]) = "None";
static char const CAL_PGM_DEF(parityEven[]) = "Even";
static char const CAL_PGM_DEF(parityOdd[])  =  "Odd ";

static char const CAL_PGM_DEF(* CAL_PGM_DEF(parityStrings[PARITY_COUNT])) = {
	parityNone,
	parityEven,
	parityOdd
};

static UART_parityMode_t const CAL_PGM_DEF(parityValues[PARITY_COUNT]) = {
	UART_PARITY_NONE,
	UART_PARITY_EVEN,
	UART_PARITY_ODD
};

static char const CAL_PGM_DEF(interfaceLogic[]) = "Logic";
static char const CAL_PGM_DEF(interfaceRS232[]) = "RS232";

static char const CAL_PGM_DEF(* CAL_PGM_DEF(interfaceStrings[INTERFACE_COUNT])) = {
	interfaceLogic,
	interfaceRS232
};

static char const CAL_PGM_DEF(buttonApply[])  = "Apply";
static char const CAL_PGM_DEF(buttonCancel[]) = "Cancel";

static char const CAL_PGM_DEF( * CAL_PGM_DEF(buttonStrings[BUTTON_COUNT])) = {
	buttonApply,
	buttonCancel
};


static WIDGETS_integer_t CONFIG_baudSelection = 0;
static WIDGETS_integer_t CONFIG_databitSelection = 8;
static WIDGETS_integer_t CONFIG_stopbitSelection = 1;
static WIDGETS_integer_t CONFIG_paritySelection = 0;
static WIDGETS_integer_t CONFIG_interfaceSelection = 0;


void ConfigUart( void )
{
	FORMS_form_t * form = MEM_ALLOC( FORMS_form_t );
	WIDGETS_StaticText_t * titleCtrl = MEM_ALLOC( WIDGETS_StaticText_t );
	WIDGETS_Separator_t * topSep = MEM_ALLOC( WIDGETS_Separator_t );
	WIDGETS_SpinCtrl_t * baudCtrl = MEM_ALLOC( WIDGETS_SpinCtrl_t );
	WIDGETS_SpinCtrl_t * databitCtrl = MEM_ALLOC( WIDGETS_SpinCtrl_t );
	WIDGETS_SpinCtrl_t * stopbitCtrl = MEM_ALLOC( WIDGETS_SpinCtrl_t );
	WIDGETS_SpinCtrl_t * parityCtrl = MEM_ALLOC( WIDGETS_SpinCtrl_t );
	WIDGETS_SpinCtrl_t * interfaceCtrl = MEM_ALLOC( WIDGETS_SpinCtrl_t );
	WIDGETS_Separator_t * bottomSep = MEM_ALLOC( WIDGETS_Separator_t );
	WIDGETS_ButtonRow_t * buttonRow = MEM_ALLOC( WIDGETS_ButtonRow_t );

	if ((form == NULL) || (titleCtrl == NULL) || (topSep == NULL) ||
			(baudCtrl == NULL) || (databitCtrl == NULL) || (stopbitCtrl == NULL) ||
			(parityCtrl == NULL) || (interfaceCtrl == NULL) || (bottomSep == NULL) ||
			(buttonRow == NULL)) {
		POPUP_MsgBox( 10, 2, 6, "Not enough\r\nmemory!", NULL );
	} else {
		FORMS_Init( form, true );
		
		WIDGETS_StaticText_Init( titleCtrl, 0, 1, true, false );
		FORMS_SetCaption_F( &titleCtrl->element, titleString );
		FORMS_AddBottomElement( form, &titleCtrl->element );
	
		WIDGETS_Separator_Init( topSep );
		FORMS_AddBottomElement( form, &topSep->element );
		
		WIDGETS_SpinCtrl_Init( baudCtrl,
				0, // Id doesn't care.
				&CONFIG_baudSelection,
				WIDGETS_SpinCtrl_DrawString,
				0, // from 0...
				BAUD_COUNT - 1, // to last baud
				1, // step size = 1
				BAUD_WIDTH ); // data field is 6 chars.
		FORMS_SetCaption_F( &baudCtrl->element, baudCaption );
		WIDGETS_SpinCtrl_SetStringList_FF( baudCtrl, baudStrings );
		FORMS_AddBottomElement( form, &baudCtrl->element );	
	
		WIDGETS_SpinCtrl_Init( databitCtrl,
				0, // Id doesn't care.
				&CONFIG_databitSelection,
				WIDGETS_SpinCtrl_DrawDec,
				5, // from 5...
				9, // ...to 9
				1, // step size = 1
				1 ); // data field is 1 char.
		FORMS_SetCaption_F( &databitCtrl->element, databitCaption );
		FORMS_AddBottomElement( form, &databitCtrl->element );	
	
		WIDGETS_SpinCtrl_Init( stopbitCtrl,
				0, // Id doesn't care.
				&CONFIG_stopbitSelection,
				WIDGETS_SpinCtrl_DrawDec,
				1, // from 1...
				2, // ...to 2
				1, // step size = 1
				1 ); // data field is 1 char.
		FORMS_SetCaption_F( &stopbitCtrl->element, stopbitCaption );
		FORMS_AddBottomElement( form, &stopbitCtrl->element );	
	
		WIDGETS_SpinCtrl_Init( parityCtrl,
				0, // Id doesn't care.
				&CONFIG_paritySelection,
				WIDGETS_SpinCtrl_DrawString,
				0, // from 0...
				PARITY_COUNT - 1, // to last baud
				1, // step size = 1
				PARITY_WIDTH ); // data field width.
		FORMS_SetCaption_F( &parityCtrl->element, parityCaption );
		WIDGETS_SpinCtrl_SetStringList_FF( parityCtrl, parityStrings );
		FORMS_AddBottomElement( form, &parityCtrl->element );	
	
		WIDGETS_SpinCtrl_Init( interfaceCtrl,
				0, // Id doesn't care.
				&CONFIG_interfaceSelection,
				WIDGETS_SpinCtrl_DrawString,
				0, // from 0...
				INTERFACE_COUNT - 1, // to last baud
				1, // step size = 1
				INTERFACE_WIDTH ); // data field width.
		FORMS_SetCaption_F( &interfaceCtrl->element, interfaceCaption );
		WIDGETS_SpinCtrl_SetStringList_FF( interfaceCtrl, interfaceStrings );
		FORMS_AddBottomElement( form, &interfaceCtrl->element );	
	
		WIDGETS_Separator_Init( bottomSep );
		FORMS_AddBottomElement( form, &bottomSep->element );
		
		WIDGETS_id_t const buttonIDs[] = { DIALOG_OK, DIALOG_CANCEL };
		WIDGETS_ButtonRow_Init( buttonRow, buttonIDs, 0, 2 );
		WIDGETS_ButtonRow_SetStringList_FF( buttonRow, buttonStrings );
		WIDGETS_ButtonRow_SetOnClick( buttonRow, DIALOG_OnClickInModal );
		FORMS_AddBottomElement( form, &buttonRow->element );
	
		FORMS_SetFocusedElement( form, &baudCtrl->element );
		WIDGETS_id_t result = DIALOG_RunModal( form );
		
		if (result == DIALOG_OK) {
			if ((CONFIG_databitSelection != 7) && (CONFIG_databitSelection != 8)) {
				POPUP_MsgBox( 14, 4, 6,
						"Sorry. Only 7-"
						"and 8-bit data"
						"can be used at"
						"this time.", form );
			} else {
				PRR1 &= ~(1 << PRUSART1); // Enable USART1.

				///mt UART_SetBaudRateValue( baudValues[CONFIG_baudSelection] );
				UART_SetBaudRateValue( CAL_pgm_read_word(&baudValues[CONFIG_baudSelection]) );
				///mt UART_SetParityMode( parityValues[CONFIG_paritySelection] );
				UART_SetParityMode( CAL_pgm_read_enum(&parityValues[CONFIG_paritySelection]) );
				UART_SetDataBits( CONFIG_databitSelection );
				UART_SetStopBits( CONFIG_stopbitSelection );
				
				if (CONFIG_interfaceSelection == 0) { // Logic
					CONFIG_SET_LOGIC_INTERFACE();
				}
				else if (CONFIG_interfaceSelection == 1) { // RS232
					CONFIG_SET_RS232_INTERFACE();
				}
			}
		}
		
		FORMS_CleanUp();
	}

	MEM_FREE( form );
	MEM_FREE( titleCtrl );
	MEM_FREE( topSep );
	MEM_FREE( baudCtrl );
	MEM_FREE( databitCtrl );
	MEM_FREE( stopbitCtrl );
	MEM_FREE( parityCtrl );
	MEM_FREE( interfaceCtrl );
	MEM_FREE( bottomSep );
	MEM_FREE( buttonRow );
}
