/* SD card command definitions file */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef SDCMD_H
#define SDCMD_H

#include <stdint.h>

/* card state   host state */
typedef enum {
  SD_Inactive=0,/* inactive */
  SD_Idle,      /* card identification */
  SD_Ready,     /* card identification */
  SD_Identify,  /* card identification */
  SD_Standby,   /* data transfer */
  SD_Transfer,  /* data transfer */
  SD_SendData,  /* data transfer */
  SD_RecvData,  /* data transfer */
  SD_Program,   /* data transfer */
  SD_Disconnect,/* data transfer */
} SD_State_t;

#define CMD(c) (((c) & 0x3f) | 0x40)

/* commands are transmitted MSB first */

#define SD_GO_IDLE_STATE CMD(0)
#define SD_SEND_IF_COND  CMD(8)
#define SD_SEND_CSD      CMD(9)
#define SD_SEND_CID      CMD(10)
#define SD_STOP_TRANSMISSION CMD(12)
#define SD_SEND_STATUS   CMD(13)
#define SD_SET_BLOCKLEN  CMD(16)
#define SD_READ_SINGLE_BLOCK CMD(17)
#define SD_READ_MULTIPLE_BLOCK CMD(18)
#define SD_WRITE_BLOCK   CMD(24)
#define SD_WRITE_MULTIPLE_BLOCK CMD(25)
#define SD_PROGRAM_CSD   CMD(27)
#define SD_APP_CMD       CMD(55)
#define SD_READ_OCR      CMD(58)
#define SD_CRC_ON_OFF    CMD(59)

#define SD_A_SEND_OP_COND CMD(41)

#define SDTOKEN_START          (0b11111110)
#define SDTOKEN_START_MULTIPLE (0b11111100)
#define SDTOKEN_STOP_MULTIPLE  (0b11111101)

typedef struct sd_csd_data_s {
  uint8_t csd_structure; /* 2bit csd structure version|6bit reserved */
#define CSD_STRUCTURE(csd)          ((csd).csd_structure >> 6)
  uint8_t taac; /* data read access time-1 */
  uint8_t nsac; /* data read access time-2 */
  uint8_t tran_speed; /* max data transfer rate */
#define VAL_CSD_TRAN_UNIT_RAW(v)    ((v) & 7)
#define VAL_CSD_TRAN_TIME_RAW(v)    (((v) >> 3) & 0x0f)
#define VAL_CSD_TRAN_UNIT_CONV(u)   (u == 0 ? 100e3 : u == 1 ? 1e6 :	\
				     u == 2 ? 10e6 : u == 3 ? 100e6 :	\
				     0)
#define VAL_CSD_TRAN_TIME_CONV(t)   (t == 1 ? 1.0 : t == 2 ? 1.2 :	\
				     t == 3 ? 1.3 : t == 4 ? 1.5 :	\
				     t == 5 ? 2.0 : t == 6 ? 2.5 :	\
				     t == 7 ? 3.0 : t == 8 ? 3.5 :	\
				     t == 9 ? 4.0 : t == 10 ? 4.5 :	\
				     t == 11 ? 5.0 : t == 12 ? 5.5 :	\
				     t == 13 ? 6.0 : t == 14 ? 7.0 :	\
				     t == 15 ? 8.0 : 0)
#define VAL_CSD_TRAN_SPEED(v) (VAL_CSD_TRAN_TIME_CONV(VAL_CSD_TRAN_TIME_RAW(v))	\
			       * VAL_CSD_TRAN_UNIT_CONV(VAL_CSD_TRAN_UNIT_RAW(v)))
#define CSD_TRAN_SPEED(csd)         VAL_CSD_TRAN_SPEED((csd.tran_speed))
  uint8_t ccc_read_bl_len[2]; /* card command classes
			       * max read data block len */
#define CSD_CCC(csd)                (((uint16_t)(csd).ccc_read_bl_len[0] << 4) \
				     | ((uint16_t)((csd).ccc_read_bl_len[1]) >> 4))
#define CSD_READ_BL_LEN(csd)        ((csd).ccc_read_bl_len[1] & 0xf)
  uint8_t flags1[10];
#define CSD_READ_BL_PARTIAL(csd)    ((csd).flags1[0]>>7)
#define CSD_WRITE_BLK_MISALIGN(csd) (((csd).flags1[0]>>6) & 0x1)
#define CSD_READ_BLK_MISALIGN(csd)  (((csd).flags1[0]>>5) & 0x1)
#define CSD_DSR_IMP(csd)            (((csd).flags1[0]>>4) & 0x1)
#define CSD1_C_SIZE(csd)            (((uint16_t)((csd).flags1[0] & 0x3) << 10) \
				     | ((uint16_t)((csd).flags1[1]) << 2) \
				     | ((uint16_t)((csd).flags1[2]) >> 6))
#define CSD2_C_SIZE(csd)            (((uint32_t)((csd).flags1[1] & 0x3f) << 16)\
				     | ((uint16_t)((csd).flags1[2]) << 8) \
				     | ((uint16_t)((csd).flags1[3]) ))
#define CSD_VDD_R_CURR_MIN(csd)     (((csd).flags1[2]>>3) & 0x7)
#define CSD_VDD_R_CURR_MAX(csd)     (((csd).flags1[2]) & 0x7)
#define CSD_VDD_W_CURR_MIN(csd)     (((csd).flags1[3]>>5) & 0x7)
#define CSD_VDD_W_CURR_MAX(csd)     (((csd).flags1[3]>>2) & 0x7)
#define CSD_C_SIZE_MULT(csd)        ((((csd).flags1[3] & 0x3) << 1) \
				     | ((csd).flags1[4]>>7))
#define CSD_ERASE_BLK_EN(csd)       (((csd).flags1[4]>>6) & 0x1)
#define CSD_SECTOR_SIZE(csd)        (((csd).flags1[4] & 0x3f) \
				     | ((csd).flags1[5]>>7))
#define CSD_WP_GRP_SIZE(csd)        ((csd).flags1[5] & 0x7f)
#define CSD_WP_GRP_ENABLE(csd)      ((csd).flags1[6]>>7)
#define CSD_R2W_FACTOR(csd)         (((csd).flags1[6]>>2) & 0x7)
#define CSD_WRITE_BL_LEN(csd)       (((csd).flags1[6] & 0x3) \
				     | ((csd).flags1[7]>>6))
#define CSD_WRITE_BL_PARTIAL(csd)   (((csd).flags1[7]>>5) & 0x1)
#define CSD_FILE_FORMAT_GRP(csd)    ((csd).flags1[8]>>7)
#define CSD_COPY(csd)               (((csd).flags1[8]>>6) & 0x1)
#define CSD_PERM_WRITE_PROTECT(csd) (((csd).flags1[8]>>5) & 0x1)
#define CSD_TMP_WRITE_PROTECT(csd)  (((csd).flags1[8]>>4) & 0x1)
#define CSD_FILE_FORMAT(csd)        (((csd).flags1[8]>>2) & 0x3)
#define CSD_CRC(csd)                ((csd).flags1[9]>>1)
} sd_csd_data_t;

/* card capacity in kB */
/* for SD 1.x: C_SIZE*2^(C_SIZE_MULT+2)*2^READ_BL_LEN bytes */
#define CSD1_CARD_CAPACITY_KB(csd)					\
  ((uint32_t)(CSD1_C_SIZE(csd)+1)<<(CSD_C_SIZE_MULT(csd)+2+CSD_READ_BL_LEN(csd)-10))
/* for SD 2.0: C_SIZE*512 kB */
#define CSD2_CARD_CAPACITY_KB(csd)		\
  ((uint32_t)(CSD2_C_SIZE(csd)+1)<<9)
/* assume 2.x when not 1.x */
#define CSD_CARD_CAPACITY_KB(csd)		\
  (CSD_STRUCTURE(csd) == 0 ? CSD1_CARD_CAPACITY_KB(csd) \
   : CSD2_CARD_CAPACITY_KB(csd))

typedef struct sd_cid_data_s {
  uint8_t mid;    /* manufacturer ID */
  uint8_t oid[2]; /* OEM/Application ID */
  uint8_t pnm[5]; /* product name */
  uint8_t prv;    /* product revision */
  uint32_t psn;   /* product serial# */
  uint8_t mdt[2]; /* 4bit reserved|manufacturing date */
  uint8_t crc;    /* CRC7 checksum|1 */
} sd_cid_data_t;

/* blocklen for SD transfers
 * NOTE: we have no more than 2^16 bytes address space,
 * so we will not be able to use all 32 bits defined in SD protocol anyway
 */
typedef uint16_t sd_blen_t;

/* CRC: CCITT-16 */
/* compute CRC for 136bit response block */
extern uint16_t sd_resp16_crc (uint8_t sd_response[]);
#if 0
extern void sd_send_cmd (uint8_t cmd, uint32_t arg);
extern void sd_send_acmd (uint8_t acmd, uint32_t arg);
#endif

extern uint16_t sd_init_card (void(*print)(const char *));
extern uint8_t sd_read_block (uint8_t *data, uint32_t address);

#endif
