/* SPI access functions */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "spi.h"

static void spi_calc_speed (uint8_t div, uint8_t *spr, uint8_t *spi2x)
{
  /* round up div to next power of 2 between 2 and 128 */
  if (div > 16) {
    if (div > 32) {
      if (div > 64) {
	*spi2x = 0*_BV(SPI2X);
	*spr = 1*_BV(SPR1)|1*_BV(SPR0);
	return;
      } else {
	*spi2x = 0*_BV(SPI2X);
	*spr = 1*_BV(SPR1)|0*_BV(SPR0);
	return;
      }
    } else {
      *spi2x = 1*_BV(SPI2X);
      *spr = 1*_BV(SPR1)|0*_BV(SPR0);
      return;
    }
  } else {
    if (div > 4) {
      if (div > 8) {
	*spi2x = 0*_BV(SPI2X);
	*spr = 0*_BV(SPR1)|1*_BV(SPR0);
	return;
      } else {
	*spi2x = 1*_BV(SPI2X);
	*spr = 0*_BV(SPR1)|1*_BV(SPR0);
      }
    } else {
      if (div > 2) {
	*spi2x = 0*_BV(SPI2X);
	*spr = 0*_BV(SPR1)|0*_BV(SPR0);
	return;
      } else {
	*spi2x = 1*_BV(SPI2X);
	*spr = 0*_BV(SPR1)|0*_BV(SPR0);
	return;
      }
    }
  }
}

void spi_set_speed (uint8_t div)
{
  uint8_t spr, spi2x;

  spi_calc_speed (div, &spr, &spi2x);

  SPSR = (SPSR & ~_BV(SPI2X)) | spi2x;
  SPCR = (SPCR & ~_BV(SPR1) & ~_BV(SPR0)) | spr;
}

void spi_init (uint8_t div)
{
  uint8_t spr, spi2x;

  spi_calc_speed (div, &spr, &spi2x);

  /* power-up SPI */
  PRR0 &= ~_BV(PRSPI);

  /* outputs: SS, MOSI, SCK */
  DDR_SPI = (DDR_SPI | _BV(DR_SPI_SS) | _BV(DR_SPI_SCK) | _BV(DR_SPI_MOSI))
  /* input: MISO */
    & ~(_BV(DR_SPI_MISO));

  /* pull SS high - deselect slave */
  PORT_SPI |= _BV(PORT_SPI_SS);

  SPSR = spi2x;
  SPCR = _BV(SPE)  /* enable */
    | 0*_BV(DORD)  /* MSB first */
    | _BV(MSTR)    /* master */
    | 0*_BV(CPOL)  /* mode 0 */
    | 0*_BV(CPHA)
    | spr
    ;
}
