/* parameters defined by NMEA 0183 */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef NMEA_READER_H
#define NMEA_READER_H

#include <stdint.h>
#include "gps.h"

#define NMEA_SPEED (9600)
#define NMEA_SERIALMODE 8N1

#define NMEA_MSG_START '$'
#define NMEA_MSG_DELIM ','
#define NMEA_MSG_STOP  '*'
#define NMEA_MSG_END1  '\r'
#define NMEA_MSG_END2  '\n'


typedef enum {
  NPF_INVALID = 0,
  NPF_VALID = 1,
  NPF_DGPS_VALID = 2,
  NPF_DEADRECKONING = 6,
} nmea_positionfix_t;

typedef enum {
  UNIT_METER,
  UNIT_FOOT,
} unit_t;

#define NUMCHANNELS (12)

typedef struct nmea_data_s {
  uint8_t csum, csum_read;
  gps_time_t time;
  gps_date_t date;
  uint32_t latitude;
  uint32_t longitude;
  uint32_t altitude;
  uint32_t geoid_separation;
  uint16_t dgps_age;
  uint16_t dgps_refid;
  uint16_t quality;
  uint32_t pdop, hdop, vdop;
  uint8_t mode1, mode2;
  uint16_t numsats;
  struct {
    uint8_t svid;
    uint16_t ele;
    uint16_t azi;
    uint16_t snr;
  } satposition[NUMCHANNELS];
  uint8_t ok_to_send;
  struct {
    uint8_t hours;
    uint8_t minutes;
  } utc_offset;
  nmea_positionfix_t positionfix:3;
  unit_t altitude_unit:2;
  unit_t geoid_separation_unit:2;
  uint8_t latitude_sign:1;
  uint8_t longitude_sign:1;
  struct {
    uint8_t valid:1;
    uint8_t error:1;
    uint8_t latitude:1;
    uint8_t latitude_sign:1;
    uint8_t longitude:1;
    uint8_t longitude_sign:1;
    uint8_t altitude:1;
    uint8_t altitude_unit:1;
    uint8_t geoid_separation:1;
    uint8_t geoid_separation_unit:1;
    uint8_t time:1;
    uint8_t year4:1;
    uint8_t year2:1;
    uint8_t month:1;
    uint8_t day:1;
    uint8_t positionfix:1;
    uint8_t pdop:1, hdop:1, vdop:1;
    uint8_t dgps_age:1;
    uint8_t dgps_refid:1;
    uint8_t numsats:1;
    uint8_t ok_to_send:1;
    uint8_t utc_offset_hours:1;
    uint8_t utc_offset_minutes:1;
    uint8_t mode1:1, mode2:1;
    uint8_t gsv_num_msg:2;
    uint8_t gsv_msg_num:2;
    uint8_t gsv_satnum:4;
    uint16_t svid:NUMCHANNELS;
    uint16_t satpos_ele:NUMCHANNELS;
    uint16_t satpos_azi:NUMCHANNELS;
    uint16_t satpos_snr:NUMCHANNELS;
  } status;
} nmea_data_t;

/* calculate checksum for LEN bytes long string S */
uint8_t nmea_checksum (const char * s, uint8_t len);
uint8_t nmea_init (void);

void nmea_serial_in_handler(uint8_t b);

extern void (*nmea_start_callback)(const char *);

#endif
