/* serial communications routines */
/* Copyright (C) 2009 Holger Dietze
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "config.h"
#include "serial.h"
#include "events.h"

#define BAUD_TOL (1)
static const struct serial_param_s {
  uint16_t baud;
  uint16_t rate;
  uint8_t use_2x;
} serial_param[] = {
#include "baud-params.h"
  {0, 0, 0} /* END MARKER */
};

void
serial_init (uint16_t speed)
{
  const struct serial_param_s *p;

  for (p = serial_param; p->baud && (p->baud != speed); p++);

  if (p->baud) {
    UBRR0H = p->rate >> 8;
    UBRR0L = p->rate & 0xff;
    if (p->use_2x)
      UCSR0A |= _BV(U2X0);
    else
      UCSR0A &= ~_BV(U2X0);

    /* enable receiver and RX interrupt */
    UCSR0B = _BV(RXCIE0) | _BV(RXEN0);
    /* frame format: 8N1 */
    UCSR0C = _BV(UCSZ01) | _BV(UCSZ00);
  }
}

ISR(USART_RX_vect)
{
  while (UCSR0A & _BV(RXC0)) /* try to read as much as possible in one go */
    put_serial_in_event (UDR0);
}
